@@ -1,7 +1,7 @@
 # SPDX-FileCopyrightText: 2023 geisserml <geisserml@gmail.com>
 # SPDX-License-Identifier: Apache-2.0 OR BSD-3-Clause
 
-from pypdfium2._helpers._internal import consts
+import pypdfium2.internal as pdfium_i
 from pypdfium2._cli._parsers import (
     add_input,
     add_n_digits,
@@ -29,24 +29,13 @@ def main(args):
     )
     
     for item in toc:
-        
-        if item.n_kids == 0:
-            state = "*"
-        elif item.is_closed:
-            state = "-"
-        else:
-            state = "+"
-        
-        if item.page_index is None:
-            target = "?"
-        else:
-            target = item.page_index + 1
-        
+        state = "*" if item.n_kids == 0 else "-" if item.is_closed else "+"
+        target = "?" if item.page_index is None else item.page_index+1
         print(
             "    " * item.level +
             "[%s] %s -> %s  # %s %s" % (
                 state, item.title, target,
-                consts.ViewmodeToStr.get(item.view_mode),
+                pdfium_i.ViewmodeToStr.get(item.view_mode),
                 round_list(item.view_pos, args.n_digits),
             )
         )
