@@ -1,12 +1,12 @@
 # SPDX-FileCopyrightText: 2023 geisserml <geisserml@gmail.com>
 # SPDX-License-Identifier: Apache-2.0 OR BSD-3-Clause
 
-__all__ = ["PdfAttachment"]
+__all__ = ("PdfAttachment", )
 
 import ctypes
 import pypdfium2.raw as pdfium_c
 from pypdfium2._helpers.misc import PdfiumError
-from pypdfium2._helpers._internal.bases import AutoCastable
+import pypdfium2.internal as pdfium_i
 
 
 def _encode_key(key):
@@ -16,7 +16,7 @@ def _encode_key(key):
         raise TypeError(f"Key must be str, but {type(key).__name__} was given.")
 
 
-class PdfAttachment (AutoCastable):
+class PdfAttachment (pdfium_i.AutoCastable):
     """
     Attachment helper class.
     See PDF 1.7, Section 7.11 "File Specifications".
@@ -36,8 +36,7 @@ class PdfAttachment (AutoCastable):
     
     
     def __init__(self, raw, pdf):
-        self.raw = raw
-        self.pdf = pdf
+        self.raw, self.pdf = raw, pdf
     
     
     def get_name(self):
@@ -66,9 +65,9 @@ class PdfAttachment (AutoCastable):
         
         buffer = ctypes.create_string_buffer(n_bytes)
         out_buflen = ctypes.c_ulong()
-        success = pdfium_c.FPDFAttachment_GetFile(self, buffer, n_bytes, out_buflen)
+        ok = pdfium_c.FPDFAttachment_GetFile(self, buffer, n_bytes, out_buflen)
         out_buflen = out_buflen.value
-        if not success:
+        if not ok:
             raise PdfiumError("Failed to extract attachment (error status).")
         if n_bytes < out_buflen:
             raise PdfiumError(f"Failed to extract attachment (expected {n_bytes} bytes, but got {out_buflen}).")
@@ -86,8 +85,8 @@ class PdfAttachment (AutoCastable):
             data (bytes | ctypes.Array):
                 New file data for the attachment. May be any data type that can be implicitly converted to :class:`~ctypes.c_void_p`.
         """
-        success = pdfium_c.FPDFAttachment_SetFile(self, self.pdf, data, len(data))
-        if not success:
+        ok = pdfium_c.FPDFAttachment_SetFile(self, self.pdf, data, len(data))
+        if not ok:
             raise PdfiumError("Failed to set attachment data.")
     
     
@@ -138,6 +137,6 @@ class PdfAttachment (AutoCastable):
         """
         enc_value = (value + "\x00").encode("utf-16-le")
         enc_value_ptr = ctypes.cast(enc_value, pdfium_c.FPDF_WIDESTRING)
-        success = pdfium_c.FPDFAttachment_SetStringValue(self, _encode_key(key), enc_value_ptr)
-        if not success:
+        ok = pdfium_c.FPDFAttachment_SetStringValue(self, _encode_key(key), enc_value_ptr)
+        if not ok:
             raise PdfiumError(f"Failed to set attachment param '{key}' to '{value}'.")
