@@ -1,7 +1,7 @@
 # SPDX-FileCopyrightText: 2023 geisserml <geisserml@gmail.com>
 # SPDX-License-Identifier: Apache-2.0 OR BSD-3-Clause
 
-__all__ = ["PdfDocument", "PdfFormEnv", "PdfXObject", "PdfOutlineItem"]
+__all__ = ("PdfDocument", "PdfFormEnv", "PdfXObject", "PdfOutlineItem")
 
 import os
 import ctypes
@@ -12,18 +12,21 @@ from collections import namedtuple
 from concurrent.futures import ProcessPoolExecutor
 
 import pypdfium2.raw as pdfium_c
-from pypdfium2.version import V_PDFIUM_IS_V8
+from pypdfium2.version import (
+    V_LIBPDFIUM,
+    V_BUILDNAME,
+    V_PDFIUM_IS_V8,
+)
 from pypdfium2._helpers.misc import PdfiumError
 from pypdfium2._helpers.page import PdfPage
 from pypdfium2._helpers.pageobjects import PdfObject
 from pypdfium2._helpers.attachment import PdfAttachment
-from pypdfium2._helpers._internal import consts, utils
-from pypdfium2._helpers._internal.bases import AutoCloseable
+import pypdfium2.internal as pdfium_i
 
 logger = logging.getLogger(__name__)
 
 
-class PdfDocument (AutoCloseable):
+class PdfDocument (pdfium_i.AutoCloseable):
     """
     Document helper class.
     
@@ -54,12 +57,7 @@ class PdfDocument (AutoCloseable):
             Form env, if the document has forms and :meth:`.init_forms` was called.
     """
     
-    def __init__(
-            self,
-            input,
-            password = None,
-            autoclose = False,
-        ):
+    def __init__(self, input, password=None, autoclose=False):
         
         if isinstance(input, str):
             input = Path(input)
@@ -71,9 +69,10 @@ class PdfDocument (AutoCloseable):
         self._input = input
         self._password = password
         self._autoclose = autoclose
-        
         self._data_holder = []
         self._data_closer = []
+        
+        # question: can we make attributes like formenv effectively immutable for the caller?
         self.formenv = None
         
         if isinstance(self._input, pdfium_c.FPDF_DOCUMENT):
@@ -83,7 +82,19 @@ class PdfDocument (AutoCloseable):
             self._data_holder += to_hold
             self._data_closer += to_close
         
-        AutoCloseable.__init__(self, self._close_impl, self._data_holder, self._data_closer)
+        super().__init__(PdfDocument._close_impl, self._data_holder, self._data_closer)
+    
+    
+    def __repr__(self):
+        if isinstance(self._input, Path):
+            input_r = repr( str(self._input) )
+        elif isinstance(self._input, bytes):
+            input_r = f"<bytes object at {hex(id(self._input))}>"
+        elif isinstance(self._input, pdfium_c.FPDF_DOCUMENT):
+            input_r = f"<FPDF_DOCUMENT at {hex(id(self._input))}>"
+        else:
+            input_r = repr(self._input)
+        return f"{super().__repr__()[:-1]} from {input_r}>"
     
     
     @property
@@ -93,7 +104,6 @@ class PdfDocument (AutoCloseable):
     
     @staticmethod
     def _close_impl(raw, data_holder, data_closer):
-        # can't close formenv here, would cause circular reference
         pdfium_c.FPDF_CloseDocument(raw)
         for data in data_holder:
             id(data)
@@ -103,12 +113,6 @@ class PdfDocument (AutoCloseable):
         data_closer.clear()
     
     
-    def close(self):
-        if self.formenv:
-            self.formenv.close()
-        AutoCloseable.close(self)
-    
-    
     def __len__(self):
         return pdfium_c.FPDF_GetPageCount(self)
     
@@ -143,21 +147,41 @@ class PdfDocument (AutoCloseable):
             before getting any page handles (due to PDFium's API).
         
         Parameters:
-            config (FPDF_FORMFILLINFO):
-                Caller-provided form config interface to use. Optional for default builds.
-                Must be given if using V8 enabled PDFium (:data:`.V_PDFIUM_IS_V8`), with all required fields implemented.
+            config (FPDF_FORMFILLINFO | None):
+                Custom form config interface to use (optional).
         """
-        if (self.get_formtype() == pdfium_c.FORMTYPE_NONE) or self.formenv:
+        
+        # safety check for older binaries (could be removed at some point)
+        # https://github.com/bblanchon/pdfium-binaries/issues/105
+        if V_PDFIUM_IS_V8 and V_BUILDNAME == "pdfium-binaries" and int(V_LIBPDFIUM) <= 5677:
+            raise RuntimeError("V8 enabled pdfium-binaries builds <= 5677 crash on init_forms().")
+        
+        formtype = self.get_formtype()
+        if formtype == pdfium_c.FORMTYPE_NONE or self.formenv:
             return
+        
         if not config:
             if V_PDFIUM_IS_V8:
-                raise RuntimeError("A caller-provided form config is required with V8 enabled PDFium.")
+                js_platform = pdfium_c.IPDF_JSPLATFORM(version=3)
+                config = pdfium_c.FPDF_FORMFILLINFO(version=2, xfa_disabled=False, jsPlatform=js_platform)
             else:
                 config = pdfium_c.FPDF_FORMFILLINFO(version=2)
+        
         raw = pdfium_c.FPDFDOC_InitFormFillEnvironment(self, config)
+        if not raw:
+            raise PdfiumError(f"Initializing form env failed for document {self}.")
         self.formenv = PdfFormEnv(raw, config, self)
+        self._add_kid(self.formenv)
+        
+        if V_PDFIUM_IS_V8 and formtype in (pdfium_c.FORMTYPE_XFA_FULL, pdfium_c.FORMTYPE_XFA_FOREGROUND):
+            ok = pdfium_c.FPDF_LoadXFA(self)
+            if not ok:
+                # always fails as of this writing, thus warn instead of raising an exception for now
+                # probably this is due to an issue with pdfium-binaries - once fixed, this may be tightened to an exception
+                logger.warning(f"Failed to load XFA for document {self}.")
     
     
+    # TODO?(v5) consider cached property
     def get_formtype(self):
         """
         Returns:
@@ -167,6 +191,7 @@ class PdfDocument (AutoCloseable):
         return pdfium_c.FPDF_GetFormType(self)
     
     
+    # TODO?(v5) consider cached property
     def get_pagemode(self):
         """
         Returns:
@@ -175,6 +200,7 @@ class PdfDocument (AutoCloseable):
         return pdfium_c.FPDFDoc_GetPageMode(self)
     
     
+    # TODO?(v5) consider cached property
     def is_tagged(self):
         """
         Returns:
@@ -185,15 +211,15 @@ class PdfDocument (AutoCloseable):
     
     def _save_to(self, buffer, version=None, flags=pdfium_c.FPDF_NO_INCREMENTAL):
         
-        c_writer = utils.get_bufwriter(buffer)
+        c_writer = pdfium_i.get_bufwriter(buffer)
         saveargs = (self, c_writer, flags)
         
         if version is None:
-            success = pdfium_c.FPDF_SaveAsCopy(*saveargs)
+            ok = pdfium_c.FPDF_SaveAsCopy(*saveargs)
         else:
-            success = pdfium_c.FPDF_SaveWithVersion(*saveargs, version)
+            ok = pdfium_c.FPDF_SaveWithVersion(*saveargs, version)
         
-        if not success:
+        if not ok:
             raise PdfiumError("Failed to save document.")
     
     
@@ -213,7 +239,7 @@ class PdfDocument (AutoCloseable):
         if isinstance(dest, (str, Path)):
             with open(dest, "wb") as buf:
                 self._save_to(buf, *args, **kwargs)
-        elif utils.is_buffer(dest, "w"):
+        elif pdfium_i.is_buffer(dest, "w"):
             self._save_to(dest, *args, **kwargs)
         else:
             raise ValueError(f"Cannot save to '{dest}'")
@@ -243,8 +269,8 @@ class PdfDocument (AutoCloseable):
             or None if the document is new or its version could not be determined.
         """
         version = ctypes.c_int()
-        success = pdfium_c.FPDF_GetFileVersion(self, version)
-        if not success:
+        ok = pdfium_c.FPDF_GetFileVersion(self, version)
+        if not ok:
             return None
         return version.value
     
@@ -327,8 +353,8 @@ class PdfDocument (AutoCloseable):
         Handles to the attachment in question received from :meth:`.get_attachment`
         must not be accessed anymore after this method has been called.
         """
-        success = pdfium_c.FPDFDoc_DeleteAttachment(self, index)
-        if not success:
+        ok = pdfium_c.FPDFDoc_DeleteAttachment(self, index)
+        if not ok:
             raise PdfiumError(f"Failed to delete attachment at index {index}.")
     
     
@@ -345,11 +371,13 @@ class PdfDocument (AutoCloseable):
         raw_page = pdfium_c.FPDF_LoadPage(self, index)
         if not raw_page:
             raise PdfiumError("Failed to load page.")
-        page = PdfPage(raw_page, self)
+        page = PdfPage(raw_page, self, self.formenv)
         
         if self.formenv:
             pdfium_c.FORM_OnAfterLoadPage(page, self.formenv)
-        # TODO attach page to formenv and add safety check on form env closing
+            self.formenv._add_kid(page)
+        else:
+            self._add_kid(page)
         
         return page
     
@@ -372,7 +400,10 @@ class PdfDocument (AutoCloseable):
         if index is None:
             index = len(self)
         raw_page = pdfium_c.FPDFPage_New(self, index, width, height)
-        return PdfPage(raw_page, self)
+        page = PdfPage(raw_page, self, None)
+        # FIXME should we make the formenv distinction for new pages, too?
+        self._add_kid(page)
+        return page
     
     
     def del_page(self, index):
@@ -401,16 +432,16 @@ class PdfDocument (AutoCloseable):
             index = len(self)
         
         if isinstance(pages, str):
-            success = pdfium_c.FPDF_ImportPages(self, pdf, pages.encode("ascii"), index)
+            ok = pdfium_c.FPDF_ImportPages(self, pdf, pages.encode("ascii"), index)
         else:
             page_count = 0
             c_pages = None
             if pages:
                 page_count = len(pages)
                 c_pages = (ctypes.c_int * page_count)(*pages)
-            success = pdfium_c.FPDF_ImportPagesByIndex(self, pdf, c_pages, page_count, index)
+            ok = pdfium_c.FPDF_ImportPagesByIndex(self, pdf, c_pages, page_count, index)
         
-        if not success:
+        if not ok:
             raise PdfiumError("Failed to import pages.")
     
     
@@ -420,8 +451,8 @@ class PdfDocument (AutoCloseable):
             (float, float): Width and height in PDF canvas units of the page at *index* (zero-based).
         """
         size = pdfium_c.FS_SIZEF()
-        success = pdfium_c.FPDF_GetPageSizeByIndexF(self, index, size)
-        if not success:
+        ok = pdfium_c.FPDF_GetPageSizeByIndexF(self, index, size)
+        if not ok:
             raise PdfiumError("Failed to get page size by index.")
         return (size.width, size.height)
     
@@ -450,17 +481,16 @@ class PdfDocument (AutoCloseable):
         Returns:
             PdfXObject: The page as XObject.
         """
-        
         raw_xobject = pdfium_c.FPDF_NewXObjectFromPage(dest_pdf, self, index)
         if raw_xobject is None:
             raise PdfiumError(f"Failed to capture page at index {index} as FPDF_XOBJECT.")
-        
-        return PdfXObject(
-            raw = raw_xobject,
-            pdf = dest_pdf,
-        )
+        xobject = PdfXObject(raw=raw_xobject, pdf=dest_pdf)
+        self._add_kid(xobject)
+        return xobject
     
     
+    # TODO(apibreak) consider switching to a wrapper class around the raw bookmark
+    # (either with getter methods, or possibly cached properties)
     def _get_bookmark(self, bookmark, level):
         
         n_bytes = pdfium_c.FPDFBookmark_GetTitle(bookmark, None, 0)
@@ -468,15 +498,11 @@ class PdfDocument (AutoCloseable):
         pdfium_c.FPDFBookmark_GetTitle(bookmark, buffer, n_bytes)
         title = buffer.raw[:n_bytes-2].decode('utf-16-le')
         
+        # TODO(apibreak) just expose count as-is rather than using two variables and doing extra work
         count = pdfium_c.FPDFBookmark_GetCount(bookmark)
-        if count < 0:
-            is_closed = True
-        elif count == 0:
-            is_closed = None
-        else:
-            is_closed = False
-        
+        is_closed = True if count < 0 else None if count == 0 else False
         n_kids = abs(count)
+        
         dest = pdfium_c.FPDFBookmark_GetDest(self, bookmark)
         page_index = pdfium_c.FPDFDest_GetDestPageIndex(self, dest)
         if page_index == -1:
@@ -498,6 +524,7 @@ class PdfDocument (AutoCloseable):
         )
     
     
+    # TODO(apibreak) change outline API (see above)
     def get_toc(
             self,
             max_depth = 15,
@@ -544,28 +571,22 @@ class PdfDocument (AutoCloseable):
     @classmethod
     def _process_page(cls, index, input_data, password, renderer, converter, pass_info, need_formenv, mk_formconfig, **kwargs):
         
-        pdf = cls(
-            input_data,
-            password = password,
-        )
+        pdf = cls(input_data, password=password)
         if need_formenv:
-            if mk_formconfig:
-                pdf.init_forms(config=mk_formconfig())
-            else:
-                pdf.init_forms()
-        page = pdf[index]
+            pdf.init_forms(config=mk_formconfig()) if mk_formconfig else pdf.init_forms()
         
+        page = pdf[index]
         bitmap = renderer(page, **kwargs)
         info = bitmap.get_info()
         result = converter(bitmap)
         
-        for g in (bitmap, page, pdf):
+        # NOTE We MUST NOT call bitmap.close() before the converted object is serialized to the main process, otherwise we would free the buffer of a foreign bitmap prematurely if the converted object references the buffer rather than owning a copy. Confirmed by POC.
+        # This is not an issue when freeing the bitmap on garbage collection, provided the converted object keeps the buffer alive.
+        
+        for g in (page, pdf):
             g.close()
         
-        if pass_info:
-            return result, info
-        else:
-            return result
+        return (result, info) if pass_info else result
     
     
     def render(
@@ -604,6 +625,8 @@ class PdfDocument (AutoCloseable):
             :data:`typing.Any`: Parameter-dependent result.
         """
         
+        # TODO(apibreak) remove mk_formconfig parameter (bloat)
+        
         if not isinstance(self._input, (Path, str)):
             raise ValueError("Can only render in parallel with file path input.")
         
@@ -632,7 +655,7 @@ class PdfDocument (AutoCloseable):
             yield from pool.map(invoke_renderer, page_indices)
 
 
-class PdfFormEnv (AutoCloseable):
+class PdfFormEnv (pdfium_i.AutoCloseable):
     """
     Form environment helper class.
     
@@ -646,10 +669,8 @@ class PdfFormEnv (AutoCloseable):
     """
     
     def __init__(self, raw, config, pdf):
-        self.raw = raw
-        self.config = config
-        self.pdf = pdf
-        AutoCloseable.__init__(self, self._close_impl, self.config, self.pdf)
+        self.raw, self.config, self.pdf = raw, config, pdf
+        super().__init__(PdfFormEnv._close_impl, self.config, self.pdf)
     
     @property
     def parent(self):  # AutoCloseable hook
@@ -662,7 +683,7 @@ class PdfFormEnv (AutoCloseable):
         pdf.formenv = None
 
 
-class PdfXObject (AutoCloseable):
+class PdfXObject (pdfium_i.AutoCloseable):
     """
     XObject helper class.
     
@@ -672,9 +693,8 @@ class PdfXObject (AutoCloseable):
     """
     
     def __init__(self, raw, pdf):
-        self.raw = raw
-        self.pdf = pdf
-        AutoCloseable.__init__(self, pdfium_c.FPDF_CloseXObject)
+        self.raw, self.pdf = raw, pdf
+        super().__init__(pdfium_c.FPDF_CloseXObject)
     
     @property
     def parent(self):  # AutoCloseable hook
@@ -688,7 +708,7 @@ class PdfXObject (AutoCloseable):
             Page objects created from an XObject remain valid after the XObject is closed.
         """
         raw_pageobj = pdfium_c.FPDF_NewFormObjectFromXObject(self)
-        return PdfObject(
+        return PdfObject(  # not a child object (see above)
             raw = raw_pageobj,
             pdf = self.pdf,
         )
@@ -696,19 +716,17 @@ class PdfXObject (AutoCloseable):
 
 def _open_pdf(input_data, password, autoclose):
     
-    to_hold = ()
-    to_close = ()
-    
+    to_hold, to_close = (), ()
     if password is not None:
-        password = (password + "\x00").encode("utf-8")
+        password = (password+"\x00").encode("utf-8")
     
     if isinstance(input_data, Path):
-        pdf = pdfium_c.FPDF_LoadDocument((str(input_data) + "\x00").encode("utf-8"), password)
+        pdf = pdfium_c.FPDF_LoadDocument((str(input_data)+"\x00").encode("utf-8"), password)
     elif isinstance(input_data, (bytes, ctypes.Array)):
         pdf = pdfium_c.FPDF_LoadMemDocument64(input_data, len(input_data), password)
         to_hold = (input_data, )
-    elif utils.is_buffer(input_data, "r"):
-        bufaccess, to_hold = utils.get_bufreader(input_data)
+    elif pdfium_i.is_buffer(input_data, "r"):
+        bufaccess, to_hold = pdfium_i.get_bufreader(input_data)
         if autoclose:
             to_close = (input_data, )
         pdf = pdfium_c.FPDF_LoadCustomDocument(bufaccess, password)
@@ -717,11 +735,12 @@ def _open_pdf(input_data, password, autoclose):
     
     if pdfium_c.FPDF_GetPageCount(pdf) < 1:
         err_code = pdfium_c.FPDF_GetLastError()
-        raise PdfiumError(f"Failed to load document (PDFium: {consts.ErrorToStr.get(err_code)}).")
+        raise PdfiumError(f"Failed to load document (PDFium: {pdfium_i.ErrorToStr.get(err_code)}).")
     
     return pdf, to_hold, to_close
 
 
+# TODO(apibreak) change outline API (see above)
 PdfOutlineItem = namedtuple("PdfOutlineItem", "level title is_closed n_kids page_index view_mode view_pos")
 """
 Bookmark information.
