@@ -1,7 +1,7 @@
 # SPDX-FileCopyrightText: 2023 geisserml <geisserml@gmail.com>
 # SPDX-License-Identifier: Apache-2.0 OR BSD-3-Clause
 
-__all__ = ["PdfPage", "PdfColorScheme"]
+__all__ = ("PdfPage", "PdfColorScheme")
 
 import math
 import ctypes
@@ -11,14 +11,13 @@ from pypdfium2._helpers.misc import PdfiumError
 from pypdfium2._helpers.bitmap import PdfBitmap
 from pypdfium2._helpers.textpage import PdfTextPage
 from pypdfium2._helpers.pageobjects import PdfObject
-from pypdfium2._helpers._internal import consts, utils
-from pypdfium2._helpers._internal.bases import AutoCloseable
+import pypdfium2.internal as pdfium_i
 
 c_float = ctypes.c_float
 logger = logging.getLogger(__name__)
 
 
-class PdfPage (AutoCloseable):
+class PdfPage (pdfium_i.AutoCloseable):
     """
     Page helper class.
     
@@ -27,21 +26,21 @@ class PdfPage (AutoCloseable):
         pdf (PdfDocument): Reference to the document this page belongs to.
     """
     
-    def __init__(self, raw, pdf):
-        self.raw = raw
-        self.pdf = pdf
-        AutoCloseable.__init__(self, self._close_impl, self.pdf)
+    def __init__(self, raw, pdf, formenv):
+        self.raw, self.pdf, self.formenv = raw, pdf, formenv
+        super().__init__(PdfPage._close_impl, self.formenv)
     
     
     @staticmethod
-    def _close_impl(raw, pdf):
-        if pdf.formenv:
-            pdfium_c.FORM_OnBeforeClosePage(raw, pdf.formenv)
+    def _close_impl(raw, formenv):
+        if formenv:
+            pdfium_c.FORM_OnBeforeClosePage(raw, formenv)
         pdfium_c.FPDF_ClosePage(raw)
     
     
     @property
     def parent(self):  # AutoCloseable hook
+        # Might want to have this point to the direct parent, i. e. (self.pdf if formenv is None else self.formenv), but this might confuse callers expecting that parent be always pdf for pages.
         return self.pdf
     
     
@@ -74,19 +73,19 @@ class PdfPage (AutoCloseable):
         Returns:
             int: Clockwise page rotation in degrees.
         """
-        return consts.RotationToDegrees[ pdfium_c.FPDFPage_GetRotation(self) ]
+        return pdfium_i.RotationToDegrees[ pdfium_c.FPDFPage_GetRotation(self) ]
     
     def set_rotation(self, rotation):
         """
         Define the absolute, clockwise page rotation (0, 90, 180, or 270 degrees).
         """
-        pdfium_c.FPDFPage_SetRotation(self, consts.RotationToConst[rotation])
+        pdfium_c.FPDFPage_SetRotation(self, pdfium_i.RotationToConst[rotation])
     
     
     def _get_box(self, box_func, fallback_func, fallback_ok):
         left, bottom, right, top = c_float(), c_float(), c_float(), c_float()
-        success = box_func(self, left, bottom, right, top)
-        if not success:
+        ok = box_func(self, left, bottom, right, top)
+        if not ok:
             return (fallback_func() if fallback_ok else None)
         return (left.value, bottom.value, right.value, top.value)
     
@@ -170,8 +169,8 @@ class PdfPage (AutoCloseable):
             The bounding box of the page (the intersection between its media box and crop box).
         """
         rect = pdfium_c.FS_RECTF()
-        success = pdfium_c.FPDF_GetPageBoundingBox(self, rect)
-        if not success:
+        ok = pdfium_c.FPDF_GetPageBoundingBox(self, rect)
+        if not ok:
             raise PdfiumError("Failed to get page bounding box.")
         return (rect.left, rect.bottom, rect.right, rect.top)
     
@@ -184,10 +183,12 @@ class PdfPage (AutoCloseable):
         Returns:
             PdfTextPage: A new text page handle for this page.
         """
-        textpage = pdfium_c.FPDFText_LoadPage(self)
-        if not textpage:
+        raw_textpage = pdfium_c.FPDFText_LoadPage(self)
+        if not raw_textpage:
             raise PdfiumError("Failed to load text page.")
-        return PdfTextPage(textpage, self)
+        textpage = PdfTextPage(raw_textpage, self)
+        self._add_kid(textpage)
+        return textpage
     
     
     def insert_obj(self, pageobj):
@@ -208,7 +209,6 @@ class PdfPage (AutoCloseable):
         if pageobj.pdf and (pageobj.pdf is not self.pdf):
             raise ValueError("The pageobject you attempted to insert belongs to a different PDF.")
         
-        # If PDFium's API allowed inserting a page object into multiple pages, our support model would need to be changed accordingly
         pdfium_c.FPDFPage_InsertObject(self, pageobj)
         pageobj._detach_finalizer()
         pageobj.page = self
@@ -218,7 +218,8 @@ class PdfPage (AutoCloseable):
     def remove_obj(self, pageobj):
         """
         Remove a page object from the page.
-        The page object must not be used afterwards, except for its ``close()`` method.
+        As of PDFium 5692, detached page objects may be only re-inserted into existing pages of the same document.
+        If the page object is not re-inserted into a page, its ``close()`` method may be called.
         
         Parameters:
             pageobj (PdfObject): The page object to remove.
@@ -228,8 +229,8 @@ class PdfPage (AutoCloseable):
             raise ValueError("The page object you attempted to remove is not part of this page.")
         
         pdfium_c.FPDFPage_RemoveObject(self, pageobj)
-        pageobj._attach_finalizer()
         pageobj.page = None
+        pageobj._attach_finalizer()
     
     
     def gen_content(self):
@@ -238,8 +239,8 @@ class PdfPage (AutoCloseable):
         
         If page content was changed, this function should be called once before saving the document or re-loading the page.
         """
-        success = pdfium_c.FPDFPage_GenerateContent(self)
-        if not success:
+        ok = pdfium_c.FPDFPage_GenerateContent(self)
+        if not ok:
             raise PdfiumError("Failed to generate page content.")
     
     
@@ -259,6 +260,8 @@ class PdfPage (AutoCloseable):
             :class:`.PdfObject`: A page object.
         """
         
+        # TODO? close skipped objects explicitly ?
+        
         if form:
             count_objects = pdfium_c.FPDFFormObj_CountObjects
             get_object = pdfium_c.FPDFFormObj_GetObject
@@ -284,6 +287,7 @@ class PdfPage (AutoCloseable):
                 pdf = self.pdf,
                 level = level,
             )
+            self._add_kid(helper_obj)
             if not filter or helper_obj.type in filter:
                 yield helper_obj
             
@@ -426,22 +430,22 @@ class PdfPage (AutoCloseable):
         )
         bitmap.fill_rect(0, 0, width, height, fill_color)
         
-        render_args = (bitmap, self, -crop[0], -crop[3], src_width, src_height, consts.RotationToConst[rotation], flags)
+        render_args = (bitmap, self, -crop[0], -crop[3], src_width, src_height, pdfium_i.RotationToConst[rotation], flags)
         
         if color_scheme is None:
             pdfium_c.FPDF_RenderPageBitmap(*render_args)
         else:
             
             pause = pdfium_c.IFSDK_PAUSE(version=1)
-            utils.set_callback(pause, "NeedToPauseNow", lambda _: False)
+            pdfium_i.set_callback(pause, "NeedToPauseNow", lambda _: False)
             
             fpdf_cs = color_scheme.convert(rev_byteorder)
             status = pdfium_c.FPDF_RenderPageBitmapWithColorScheme_Start(*render_args, fpdf_cs, pause)
             assert status == pdfium_c.FPDF_RENDER_DONE
             pdfium_c.FPDF_RenderPage_Close(self)
         
-        if may_draw_forms and self.pdf.formenv:
-            pdfium_c.FPDF_FFLDraw(self.pdf.formenv, *render_args)
+        if may_draw_forms and self.formenv:
+            pdfium_c.FPDF_FFLDraw(self.formenv, *render_args)
         
         return bitmap
 
@@ -518,18 +522,10 @@ class PdfColorScheme:
     Each color shall be provided as a list of values for red, green, blue and alpha, ranging from 0 to 255.
     """
     
-    def __init__(
-            self,
-            path_fill,
-            path_stroke,
-            text_fill,
-            text_stroke,
-        ):
+    def __init__(self, path_fill, path_stroke, text_fill, text_stroke):
         self.colors = dict(
-            path_fill_color = path_fill,
-            path_stroke_color = path_stroke,
-            text_fill_color = text_fill,
-            text_stroke_color = text_stroke,
+            path_fill_color=path_fill, path_stroke_color=path_stroke,
+            text_fill_color=text_fill, text_stroke_color=text_stroke,
         )
     
     def convert(self, rev_byteorder):
@@ -539,5 +535,5 @@ class PdfColorScheme:
         """
         fpdf_cs = pdfium_c.FPDF_COLORSCHEME()
         for key, value in self.colors.items():
-            setattr(fpdf_cs, key, utils.color_tohex(value, rev_byteorder))
+            setattr(fpdf_cs, key, pdfium_i.color_tohex(value, rev_byteorder))
         return fpdf_cs
