@@ -1,20 +1,20 @@
 # SPDX-FileCopyrightText: 2023 geisserml <geisserml@gmail.com>
 # SPDX-License-Identifier: Apache-2.0 OR BSD-3-Clause
 
-__all__ = ["PdfTextPage", "PdfTextSearcher"]
+__all__ = ("PdfTextPage", "PdfTextSearcher")
 
 import ctypes
 import logging
 import pypdfium2.raw as pdfium_c
 from pypdfium2._helpers.misc import PdfiumError
-from pypdfium2._helpers._internal.bases import AutoCloseable
+import pypdfium2.internal as pdfium_i
 
 c_double = ctypes.c_double
 
 logger = logging.getLogger(__name__)
 
 
-class PdfTextPage (AutoCloseable):
+class PdfTextPage (pdfium_i.AutoCloseable):
     """
     Text page helper class.
     
@@ -26,7 +26,7 @@ class PdfTextPage (AutoCloseable):
     def __init__(self, raw, page):
         self.raw = raw
         self.page = page
-        AutoCloseable.__init__(self, pdfium_c.FPDFText_ClosePage)
+        super().__init__(pdfium_c.FPDFText_ClosePage)
     
     @property
     def parent(self):  # AutoCloseable hook
@@ -50,7 +50,8 @@ class PdfTextPage (AutoCloseable):
         if count == -1:
             count = self.count_chars() - index
         
-        # FIXME does not take possible unicode surrogation into account
+        # NOTE apparently, pdfium treats a surrogate pair like two separate chars, so this allocates enough memory and we end up with the right result
+        # (which however is no good API design - pdfium should properly handle surrogation and the API should tell the exact amount of memory needed)
         n_bytes = count * 2
         buffer = ctypes.create_string_buffer(n_bytes+2)
         buffer_ptr = ctypes.cast(buffer, ctypes.POINTER(ctypes.c_ushort))
@@ -111,10 +112,10 @@ class PdfTextPage (AutoCloseable):
         Returns:
             int: The number of text rectangles in the given character range.
         """
-        result = pdfium_c.FPDFText_CountRects(self, index, count)
-        if result == -1:
+        n_rects = pdfium_c.FPDFText_CountRects(self, index, count)
+        if n_rects == -1:
             raise PdfiumError("Failed to count rectangles.")
-        return result
+        return n_rects
     
     
     def get_index(self, x, y, x_tol, y_tol):
@@ -151,14 +152,14 @@ class PdfTextPage (AutoCloseable):
         
         if loose:
             rect = pdfium_c.FS_RECTF()
-            success = pdfium_c.FPDFText_GetLooseCharBox(self, index, rect)
+            ok = pdfium_c.FPDFText_GetLooseCharBox(self, index, rect)
             l, b, r, t = rect.left, rect.bottom, rect.right, rect.top
         else:
             l, b, r, t = c_double(), c_double(), c_double(), c_double()
-            success = pdfium_c.FPDFText_GetCharBox(self, index, l, r, b, t)  # yes, lrbt!
+            ok = pdfium_c.FPDFText_GetCharBox(self, index, l, r, b, t)  # yes, lrbt!
             l, b, r, t = l.value, b.value, r.value, t.value
         
-        if not success:
+        if not ok:
             raise PdfiumError("Failed to get charbox.")
         
         return l, b, r, t
@@ -167,12 +168,16 @@ class PdfTextPage (AutoCloseable):
     def get_rect(self, index):
         """
         Get the bounding box of a text rectangle at the given index.
+        Note that :meth:`.count_rects` must be called once with default parameters
+        before subsequent :meth:`.get_rect` calls for this function to work (due to PDFium's API).
         
         Returns:
             Float values for left, bottom, right and top in PDF canvas units.
         """
         l, b, r, t = c_double(), c_double(), c_double(), c_double()
-        pdfium_c.FPDFText_GetRect(self, index, l, t, r, b)  # yes, ltrb!
+        ok = pdfium_c.FPDFText_GetRect(self, index, l, t, r, b)  # yes, ltrb!
+        if not ok:
+            raise PdfiumError("Failed to get rectangle. (Make sure count_rects() was called with default params once before subsequent get_rect() calls.)")
         return (l.value, b.value, r.value, t.value)
     
     
@@ -190,7 +195,8 @@ class PdfTextPage (AutoCloseable):
             match_whole_word (bool):
                 If True, substring occurrences will be ignored (e. g. `cat` would not match `category`).
             consecutive (bool):
-                TODO
+                If False (the default), :meth:`.search` will skip past the current match to look for the next match.
+                If True, parts of the previous match may be caught again (e. g. searching for `aa` in `aaaa` would match 3 rather than 2 times).
         Returns:
             PdfTextSearcher: A helper object to search text.
         """
@@ -208,11 +214,13 @@ class PdfTextPage (AutoCloseable):
         
         enc_text = (text + "\x00").encode("utf-16-le")
         enc_text_ptr = ctypes.cast(enc_text, ctypes.POINTER(ctypes.c_ushort))
-        search = pdfium_c.FPDFText_FindStart(self, enc_text_ptr, flags, index)
-        return PdfTextSearcher(search, self)
+        raw_searcher = pdfium_c.FPDFText_FindStart(self, enc_text_ptr, flags, index)
+        searcher = PdfTextSearcher(raw_searcher, self)
+        self._add_kid(searcher)
+        return searcher
 
 
-class PdfTextSearcher (AutoCloseable):
+class PdfTextSearcher (pdfium_i.AutoCloseable):
     """
     Text searcher helper class.
     
@@ -224,7 +232,7 @@ class PdfTextSearcher (AutoCloseable):
     def __init__(self, raw, textpage):
         self.raw = raw
         self.textpage = textpage
-        AutoCloseable.__init__(self, pdfium_c.FPDFText_FindClose)
+        super().__init__(pdfium_c.FPDFText_FindClose)
     
     @property
     def parent(self):  # AutoCloseable hook
@@ -232,14 +240,14 @@ class PdfTextSearcher (AutoCloseable):
     
     
     def _get_occurrence(self, find_func):
-        success = find_func(self)
-        if not success:
+        ok = find_func(self)
+        if not ok:
             return None
+        # FIXME what's the point of `count`? isn't it always equal to the length of the search text?
         index = pdfium_c.FPDFText_GetSchResultIndex(self)
         count = pdfium_c.FPDFText_GetSchCount(self)
         return index, count
     
-    
     def get_next(self):
         """
         Returns:
@@ -248,7 +256,6 @@ class PdfTextSearcher (AutoCloseable):
         """
         return self._get_occurrence(pdfium_c.FPDFText_FindNext)
     
-    
     def get_prev(self):
         """
         Returns:
